package pedido;

import funcionario.Funcionario;
import cliente.Cliente;
import produto.Produto;
import produto.RepositorioProdutos;
import produto.RepositorioProdutosLista;
import util.DadosResidenciais;
import util.Iterator;
import util.IteratorArray;


/**
 * Title: Classe Pedido
 * Description: Classe primaria Pedido
 * Copyright:    Copyright (c) 2002
 * Company: VIP team
 * @author Rafael Cavalcanti Ortolan
 * @version 1.0
 */

public class Pedido {


    // constantes de match.
    public static final int MATCH_NUMERO = 1;
    public static final int MATCH_NOME_CLIENTE = 2;
    public static final int MATCH_NOME_FUNCIONARIO  = 4;
    public static final int MATCH_TELEFONE_CLIENTE  = 8;

    private int numero;
    private float taxaEntrega;
    private Cliente cliente;
    private Funcionario funcionario;
    private RepositorioProdutos produtos;

    /**
     *   Construtor padro.
     */
    public Pedido() {
        this.setNumero(0);
        this.setTaxaEntrega(0);
        this.setCliente(new Cliente());
        this.setFuncionario(new Funcionario());
        this.setProdutos(new RepositorioProdutosLista());
    }

    /**
     *   Construtor.
     */
    public Pedido(int numPedido, Cliente cliente, Funcionario funcionario,
            RepositorioProdutos produtos, float taxaEntrega) {

        this.numero = numero;
        this.taxaEntrega = taxaEntrega;
        this.cliente = cliente;
        this.funcionario = funcionario;
        this.produtos = produtos;
    }

    /**
     *   Este mtodo devolve o numero do pedido.
     *
     * @return    nmero do pedido.
     */
    public int getNumero() {
        return this.numero;
    }

    /**
     *   Este mtodo atribui o numero do pedido.
     *
     * @param numero    Ser utilizado para a atribuicao.
     */
    public void setNumero(int numero) {
        this.numero = numero;
    }

    /**
     *   Este mtodo devolve a taxa de entrega do pedido.
     *
     * @return    taxa de entrega do pedido.
     */
    public float getTaxaEntrega() {
        return this.taxaEntrega;
    }

    /**
     * Altera a taxa de entrega do produto.
     *
     * @param valor     nova taxa de entrega do produto.
     *
     * @throws IllegalArgumentException caso a taxa de entrega seja invlida.
     */
    public void setTaxaEntrega(float valor) throws IllegalArgumentException {
        if (valor < 0) {
            throw new IllegalArgumentException("Taxa de entrega no pode "+
                                                "ser negativa.");
        } else {
            this.taxaEntrega = valor;
        }
    }

    /**
     *   Este mtodo devolve o cliente que fez o pedido.
     *
     * @return    cliente.
     */
    public Cliente getCliente() {
        return this.cliente;
    }

    /**
     *   Este mtodo atribui o cliente que fez o pedido.
     *
     * @param cliente    Ser utilizado para a atribuicao.
     */
    public void setCliente(Cliente cliente) {
        this.cliente = cliente;
    }

    /**
     *   Este mtodo altera o funcionrio que cadastrou o pedido.
     *
     * @param funcionario    Ser utilizado para a atribuicao.
     */

    public void setFuncionario(Funcionario funcionario) {
        this.funcionario = funcionario;
    }
    /**
     *   Este mtodo devolve o funcionrio que cadastrou o pedido.
     *
     * @return    funcionrio.
     */
    public Funcionario getFuncionario() {
        return this.funcionario;
    }

    /**
     *   Este mtodo devolve os produtos que tem no pedido.
     *
     * @return    produto.
     */

    public RepositorioProdutos getProdutos() {
        return this.produtos;
    }

    /**
     *   Este mtodo atribui o produto que fez o pedido.
     *
     * @param cliente    Ser utilizado para a atribuicao.
     */
    public void setProdutos(RepositorioProdutos produtos) {
        this.produtos = produtos;
    }

    /**
     * Este mtodo devolve o subtotal do pedido (soma dos preos individuais).
     *
     * @return um <i>float</i> indicando o preo total dos produtos.
     */
    public float getSubTotal() {
        float total = 0;
        int i;
        Iterator iterador = produtos.getIterator();

        while (iterador.hasNext()) {
            total += ((Produto) iterador.next()).getPreco();
        }

        return total;
    }

    /**
     *   Este mtodo devolve o total do pedido.
     *
     * @return  o total do pedido.
     */
    public float getTotal() {
        return (this.getSubTotal() + this.taxaEntrega);
    }

    /**
     * Compara dois pedidos.
     *
     * <p>O critrio de comparao utilizado  o <b>nmero</b> do pedido.
     *
     * @param o             pedido a ser comparado.
     * @return              um booleano indicando se os pedidos so equivalentes
     *
     * @throws ClassCastException caso o objeto passado no seja instncia de
     * Pedido.
     */
    public boolean equals(Object o) throws ClassCastException {
        Pedido p;

        if (o instanceof Pedido) {
            p = (Pedido) o;
            return (this.numero == p.numero);
        } else {
            throw new ClassCastException("No  instncia de Pedido");
        }
    }

    /**
     * Atualiza um pedido
     *
     * @param p     pedido que ser atualizado.
     */
    public void atualizar(Pedido p) {
        cliente = p.cliente;
        funcionario = p.funcionario;
        produtos = p.produtos;
        taxaEntrega = p.taxaEntrega;
    }

    /**
     * Retorna uma String que representa o pedido.
     *
     * @return              uma string com as caracteristicas do pedido.
     */
    public String toString() {
        int i;
        Produto temp;
        Iterator iterador = produtos.getIterator();

        String s;
        String sep = "|---------------------------------\n";

        s = ",---------------------------------\n";
        s = s + "| Pedido #" + numero + "\n";
        s = s + "| Funcionario: " + funcionario.getNome() + "\n";
        s = s + sep;
        s = s + "| Cliente: " + cliente.getNome() + "\n";
        s = s + "|\n";
        s = s + "| Rua: " + cliente.getDadosResidenciais().getEndereco() +
            "\n";
        s = s + "| Bairro: " + cliente.getDadosResidenciais().getBairro() +
            "\n";
        s = s + "| Telefone: " + cliente.getDadosResidenciais().getTelefone() +
            "\n";
        s = s + sep;
        s = s + "| Itens\t\tPreo (R$)\n";
        // imprimir a lista de produtos e seus respectivos preos:
        while (iterador.hasNext()) {
            temp = ((Produto) iterador.next());
            s = s + "| " + temp + "\t" + temp.getPreco() + "\n";
        }
        s = s + sep;
        s = s + "| Sub-total............: R$ " + getSubTotal() + "\n";
        s = s + "| Taxa de entrega......: R$ " + taxaEntrega + "\n";
        s = s + "| Total................: R$ " + getTotal() + "\n";
        s = s + "`---------------------------------\n";

        return s;
    }

    /**
     * Retorna um booleano indicando se um produto faz um <i>match</i> com outro
     * de acordo com os critrios desejados.
     *
     * @param p         o produto que ser comparado.
     * @param critrio  um inteiro indicando os critrios (atributos) que sero
     *                  usados para comparar esse produto (<i>this</i>) com o
     *                  produto fornecido.
     *                  Os critrios vlidos so MATCH_NOME, MATCH_DESCRICAO,
     *                  MATCH_TAMANHO, MATCH_TELEFONE_CLIENTE.
     *
     * @return um booleano indicando se os pedidos fazem <i>match</i>.
     * @see Produto
     * @see Lista
     */
    public boolean match(Pedido p, int criterios) {
        boolean ok = true;

        if ((criterios & MATCH_NUMERO) == MATCH_NUMERO
                && numero != p.numero) {
            ok = false;
        }

        if ((criterios & MATCH_NOME_CLIENTE) == MATCH_NOME_CLIENTE
                && !cliente.getNome().equalsIgnoreCase(p.cliente.getNome())) {
            ok = false;
        }

        if ((criterios & MATCH_NOME_FUNCIONARIO) == MATCH_NOME_FUNCIONARIO
               && !(funcionario.getNome().equalsIgnoreCase(p.funcionario.getNome()))) {
            ok = false;
        }

        if ((criterios & MATCH_TELEFONE_CLIENTE) == MATCH_TELEFONE_CLIENTE
               && !(cliente.getDadosResidenciais().getTelefone().equals(p.
               getCliente().getDadosResidenciais().getTelefone()))) {
            ok = false;
        }

        return ok;
    }

    public static void main(String[] args) {
        DadosResidenciais dados2 = new DadosResidenciais("endereco2"
            ,"telefone2","bairro2","referencia2","cidade2");
        Funcionario f = new Funcionario("Thiago", dados2, 10000
            ,Funcionario.GERENTE,"thiago","senha1");
        RepositorioProdutos l = new RepositorioProdutosLista();
        Produto p5 = new Produto("Coca-Cola","Coca-Cola"
            ,Produto.TAMANH0_LATA,1.0f);
        Produto p4 = new Produto("Calabresa","queijo, tomate, calabresa, etc."
            ,Produto.TAMANHO_MEDIO,12.0f);
        DadosResidenciais dados3 = new DadosResidenciais("endereco3"
            ,"telefone3","bairro3","referencia3","cidade3");
        Cliente c3 = new Cliente("Neto",dados3);

        l.inserir(p5);
        l.inserir(p4);

        Pedido p = new Pedido(0,c3,f,l,2);

        System.out.println(p);
    }

}