package produto;

import exception.ProdutoJaCadastradoException;
import exception.ProdutoNaoCadastradoException;

/**
 * <p>Title: CadastroProdutos</p>
 * <p>Description: Classe que controla o cadastro de produtos</p>
 * <p>Copyright: Copyright (c) 2002</p>
 *
 * @author Mayerber Carvalho Neto
 * @version 1.0
 */

public class CadastroProdutos {


    private RepositorioProdutos produtos;

    /**
     * Construtor.
     *
     * @param produtos o repositrio que guarda os produtos.
     */
    public CadastroProdutos(RepositorioProdutos produtos) {
        this.produtos = produtos;
    }

    /**
     * Lista os produtos cadastrados.
     *
     * @return      uma String que representa o cadastro.
     */
    public String toString() {
        return this.produtos.toString();
    }

    /**
     * Insere um produto <b>novo</b> no cadastro.
     *
     * @param p     novo produto.
     *
     * @throws ProdutoJaCadastradoException quando o produto j estiver no
     * cadastro.
     */
    public void inserir(Produto p) throws ProdutoJaCadastradoException {
        if (this.produtos.existe(p) == true) {
            throw new ProdutoJaCadastradoException();
        } else {
            this.produtos.inserir(p);
        }
    }

    /**
     * Remove um produto do cadastro.
     *
     * @param p     produto a ser removido.
     */
    public void remover(Produto p) throws ProdutoNaoCadastradoException {
        if (this.produtos.existe(p) == false) {
            throw new ProdutoNaoCadastradoException(p);
        } else {
            this.produtos.remover(p);
        }
    }

    /**
     * Verifica se um produto existe no cadastro.
     *
     * @param p     produto a ser verificado.
     * @return      um booleano indicando se o produto est cadastrado.
     */
    public boolean existe(Produto p) {
        return this.produtos.existe(p);
    }

    /**
     * Retorna um repositorio de produtos que atendem aos critrios definidos.
     *
     * @param p         o produto que ser usado como base da comparao.
     * @param criterios um inteiro indicando os critrios (atributos) que sero
     *                  usados para comparar o produto base com os produtos
     *                  da lista.
     *                  Os critrios vlidos so MATCH_NOME, MATCH_DESCRICAO,
     *                  MATCH_TAMANHO, MATCH_PRECO.
     *
     * @return um repositorio de produtos que atendem aos critrios definidos
     *
     * @see Produto
     * @see RepositorioProdutos
     */
    public RepositorioProdutos match(Produto p, int criterios) {
        return this.produtos.match(p, criterios);
    }

    /**
     * Atualiza um produto no repositorio
     *
     * @param p     produto que ser atualizado.
     */
    public void atualizar(Produto p) {
        this.produtos.atualizar(p);
    }

    public static void main(String[] args) {
        CadastroProdutos cad = new CadastroProdutos(
                                        new RepositorioProdutosLista());
        Produto p = new Produto("Portuguesa","tomate, cebola, etc..",
                                Produto.TAMANHO_MEDIO,14.0f);
        Produto p2 = new Produto("Mussarela","queijo, tomate, etc.",
                                 Produto.TAMANHO_GRANDE,10.0f);
        Produto p3 = new Produto("Mussarela","queijo, tomate, etc.",
                                 Produto.TAMANHO_MEDIO,11.0f);
        Produto p4 = new Produto("Calabresa","queijo, tomate, calabresa, etc.",
                                 Produto.TAMANHO_MEDIO,12.0f);

        try {
            cad.inserir(p2);
            System.out.println(cad);
            cad.inserir(p4);
            System.out.println(cad);
//            cad.remover(p4);
            System.out.println(cad);
            System.out.println("Mussarela est no cadastro? " + cad.existe(p2));
            cad.inserir(p);
            System.out.println("Portuguesa est no cadastro? " + cad.existe(p));
            p.setNome("PORTUGUESA");
            // descomente a linha a seguir para gerar uma exceo (tentar inserir o
            // mesmo produto 2x):
    //        cad.inserir(p);
            System.out.println(cad);
            System.out.println(cad.match(p,Produto.MATCH_NOME));
        } catch (ProdutoJaCadastradoException ex) {
            System.out.println(ex.getMessage());
        }
    }

}